@@ -1,4 +1,3 @@
-/*	$Id: md5.c,v 1.9 2001/11/29 19:40:37 momjian Exp $	*/
 /*	   $KAME: md5.c,v 1.3 2000/02/22 14:01:17 itojun Exp $	   */
 
 /*
@@ -28,13 +27,22 @@
  * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
+ *
+ * $PostgreSQL: pgsql/contrib/pgcrypto/md5.c,v 1.13 2005/07/11 15:07:59 tgl Exp $
  */
 
 #include "postgres.h"
-#include "px.h"
 
+#include <sys/param.h>
+
+#include "px.h"
 #include "md5.h"
 
+/* sanity check */
+#if !defined(BYTE_ORDER) || (BYTE_ORDER != LITTLE_ENDIAN && BYTE_ORDER != BIG_ENDIAN)
+#error Define BYTE_ORDER to be equal to either LITTLE_ENDIAN or BIG_ENDIAN
+#endif
+
 #define SHIFT(X, s) (((X) << (s)) | ((X) >> (32 - (s))))
 
 #define F(X, Y, Z) (((X) & (Y)) | ((~X) & (Z)))
@@ -141,7 +149,7 @@ md5_init(md5_ctxt * ctxt)
 	ctxt->md5_stb = MD5_B0;
 	ctxt->md5_stc = MD5_C0;
 	ctxt->md5_std = MD5_D0;
-	bzero(ctxt->md5_buf, sizeof(ctxt->md5_buf));
+	memset(ctxt->md5_buf, 0, sizeof(ctxt->md5_buf));
 }
 
 void
@@ -155,20 +163,18 @@ md5_loop(md5_ctxt * ctxt, const uint8 *input, unsigned len)
 
 	if (len >= gap)
 	{
-		bcopy((void *) input, (void *) (ctxt->md5_buf + ctxt->md5_i),
-			  gap);
+		memmove(ctxt->md5_buf + ctxt->md5_i, input, gap);
 		md5_calc(ctxt->md5_buf, ctxt);
 
 		for (i = gap; i + MD5_BUFLEN <= len; i += MD5_BUFLEN)
 			md5_calc((uint8 *) (input + i), ctxt);
 
 		ctxt->md5_i = len - i;
-		bcopy((void *) (input + i), (void *) ctxt->md5_buf, ctxt->md5_i);
+		memmove(ctxt->md5_buf, input + i, ctxt->md5_i);
 	}
 	else
 	{
-		bcopy((void *) input, (void *) (ctxt->md5_buf + ctxt->md5_i),
-			  len);
+		memmove(ctxt->md5_buf + ctxt->md5_i, input, len);
 		ctxt->md5_i += len;
 	}
 }
@@ -182,24 +188,21 @@ md5_pad(md5_ctxt * ctxt)
 	gap = MD5_BUFLEN - ctxt->md5_i;
 	if (gap > 8)
 	{
-		bcopy((void *) md5_paddat,
-			  (void *) (ctxt->md5_buf + ctxt->md5_i),
-			  gap - sizeof(ctxt->md5_n));
+		memmove(ctxt->md5_buf + ctxt->md5_i, md5_paddat,
+				gap - sizeof(ctxt->md5_n));
 	}
 	else
 	{
 		/* including gap == 8 */
-		bcopy((void *) md5_paddat, (void *) (ctxt->md5_buf + ctxt->md5_i),
-			  gap);
+		memmove(ctxt->md5_buf + ctxt->md5_i, md5_paddat, gap);
 		md5_calc(ctxt->md5_buf, ctxt);
-		bcopy((void *) (md5_paddat + gap),
-			  (void *) ctxt->md5_buf,
-			  MD5_BUFLEN - sizeof(ctxt->md5_n));
+		memmove(ctxt->md5_buf, md5_paddat + gap,
+				MD5_BUFLEN - sizeof(ctxt->md5_n));
 	}
 
 	/* 8 byte word */
 #if BYTE_ORDER == LITTLE_ENDIAN
-	bcopy(&ctxt->md5_n8[0], &ctxt->md5_buf[56], 8);
+	memmove(&ctxt->md5_buf[56], &ctxt->md5_n8[0], 8);
 #endif
 #if BYTE_ORDER == BIG_ENDIAN
 	ctxt->md5_buf[56] = ctxt->md5_n8[7];
@@ -220,7 +223,7 @@ md5_result(uint8 *digest, md5_ctxt * ctxt)
 {
 	/* 4 byte words */
 #if BYTE_ORDER == LITTLE_ENDIAN
-	bcopy(&ctxt->md5_st8[0], digest, 16);
+	memmove(digest, &ctxt->md5_st8[0], 16);
 #endif
 #if BYTE_ORDER == BIG_ENDIAN
 	digest[0] = ctxt->md5_st8[3];
