@@ -35,26 +35,24 @@
      XCLOSE = 261,
      XCREATE = 262,
      INSERT_TUPLE = 263,
-     STRING = 264,
-     XDEFINE = 265,
-     XDECLARE = 266,
-     INDEX = 267,
-     ON = 268,
-     USING = 269,
-     XBUILD = 270,
-     INDICES = 271,
-     UNIQUE = 272,
-     COMMA = 273,
-     EQUALS = 274,
-     LPAREN = 275,
-     RPAREN = 276,
-     OBJ_ID = 277,
-     XBOOTSTRAP = 278,
-     XSHARED_RELATION = 279,
-     XWITHOUT_OIDS = 280,
-     NULLVAL = 281,
-     low = 282,
-     high = 283
+     XDECLARE = 264,
+     INDEX = 265,
+     ON = 266,
+     USING = 267,
+     XBUILD = 268,
+     INDICES = 269,
+     UNIQUE = 270,
+     COMMA = 271,
+     EQUALS = 272,
+     LPAREN = 273,
+     RPAREN = 274,
+     OBJ_ID = 275,
+     XBOOTSTRAP = 276,
+     XSHARED_RELATION = 277,
+     XWITHOUT_OIDS = 278,
+     NULLVAL = 279,
+     low = 280,
+     high = 281
    };
 #endif
 #define CONST_P 258
@@ -63,32 +61,30 @@
 #define XCLOSE 261
 #define XCREATE 262
 #define INSERT_TUPLE 263
-#define STRING 264
-#define XDEFINE 265
-#define XDECLARE 266
-#define INDEX 267
-#define ON 268
-#define USING 269
-#define XBUILD 270
-#define INDICES 271
-#define UNIQUE 272
-#define COMMA 273
-#define EQUALS 274
-#define LPAREN 275
-#define RPAREN 276
-#define OBJ_ID 277
-#define XBOOTSTRAP 278
-#define XSHARED_RELATION 279
-#define XWITHOUT_OIDS 280
-#define NULLVAL 281
-#define low 282
-#define high 283
+#define XDECLARE 264
+#define INDEX 265
+#define ON 266
+#define USING 267
+#define XBUILD 268
+#define INDICES 269
+#define UNIQUE 270
+#define COMMA 271
+#define EQUALS 272
+#define LPAREN 273
+#define RPAREN 274
+#define OBJ_ID 275
+#define XBOOTSTRAP 276
+#define XSHARED_RELATION 277
+#define XWITHOUT_OIDS 278
+#define NULLVAL 279
+#define low 280
+#define high 281
 
 
 
 
 #if ! defined (YYSTYPE) && ! defined (YYSTYPE_IS_DECLARED)
-#line 81 "bootparse.y"
+#line 83 "bootparse.y"
 typedef union YYSTYPE {
 	List		*list;
 	IndexElem	*ielem;
@@ -97,7 +93,7 @@ typedef union YYSTYPE {
 	Oid			oidval;
 } YYSTYPE;
 /* Line 1204 of yacc.c.  */
-#line 100 "y.tab.h"
+#line 96 "y.tab.h"
 # define Int_yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
 # define YYSTYPE_IS_TRIVIAL 1
