@@ -8,23 +8,24 @@
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/optimizer/path/joinrels.c,v 1.72 2004/12/31 22:00:04 pgsql Exp $
+ *	  $PostgreSQL: pgsql/src/backend/optimizer/path/joinrels.c,v 1.76.2.2 2007/02/16 00:14:16 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
 #include "postgres.h"
 
+#include "optimizer/joininfo.h"
 #include "optimizer/pathnode.h"
 #include "optimizer/paths.h"
 
 
-static List *make_rels_by_clause_joins(Query *root,
+static List *make_rels_by_clause_joins(PlannerInfo *root,
 						  RelOptInfo *old_rel,
 						  ListCell *other_rels);
-static List *make_rels_by_clauseless_joins(Query *root,
+static List *make_rels_by_clauseless_joins(PlannerInfo *root,
 							  RelOptInfo *old_rel,
 							  ListCell *other_rels);
-static bool is_inside_IN(Query *root, RelOptInfo *rel);
+static bool has_join_restriction(PlannerInfo *root, RelOptInfo *rel);
 
 
 /*
@@ -39,27 +40,25 @@ static bool is_inside_IN(Query *root, RelOptInfo *rel);
  * joinrels[j], 1 <= j < level, is a list of rels containing j items.
  */
 List *
-make_rels_by_joins(Query *root, int level, List **joinrels)
+make_rels_by_joins(PlannerInfo *root, int level, List **joinrels)
 {
 	List	   *result_rels = NIL;
 	List	   *new_rels;
-	ListCell   *nr;
 	ListCell   *r;
 	int			k;
 
 	/*
 	 * First, consider left-sided and right-sided plans, in which rels of
-	 * exactly level-1 member relations are joined against initial
-	 * relations. We prefer to join using join clauses, but if we find a
-	 * rel of level-1 members that has no join clauses, we will generate
-	 * Cartesian-product joins against all initial rels not already
-	 * contained in it.
+	 * exactly level-1 member relations are joined against initial relations.
+	 * We prefer to join using join clauses, but if we find a rel of level-1
+	 * members that has no join clauses, we will generate Cartesian-product
+	 * joins against all initial rels not already contained in it.
 	 *
-	 * In the first pass (level == 2), we try to join each initial rel to
-	 * each initial rel that appears later in joinrels[1].	(The
-	 * mirror-image joins are handled automatically by make_join_rel.)	In
-	 * later passes, we try to join rels of size level-1 from
-	 * joinrels[level-1] to each initial rel in joinrels[1].
+	 * In the first pass (level == 2), we try to join each initial rel to each
+	 * initial rel that appears later in joinrels[1].  (The mirror-image joins
+	 * are handled automatically by make_join_rel.)  In later passes, we try
+	 * to join rels of size level-1 from joinrels[level-1] to each initial rel
+	 * in joinrels[1].
 	 */
 	foreach(r, joinrels[level - 1])
 	{
@@ -73,39 +72,27 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 			other_rels = list_head(joinrels[1]);		/* consider all initial
 														 * rels */
 
-		if (old_rel->joininfo != NIL)
+		if (old_rel->joininfo != NIL || has_join_restriction(root, old_rel))
 		{
 			/*
-			 * Note that if all available join clauses for this rel
-			 * require more than one other rel, we will fail to make any
-			 * joins against it here.  In most cases that's OK; it'll be
-			 * considered by "bushy plan" join code in a higher-level pass
-			 * where we have those other rels collected into a join rel.
+			 * Note that if all available join clauses for this rel require
+			 * more than one other rel, we will fail to make any joins against
+			 * it here.  In most cases that's OK; it'll be considered by
+			 * "bushy plan" join code in a higher-level pass where we have
+			 * those other rels collected into a join rel.
+			 *
+			 * See also the last-ditch case below.
 			 */
 			new_rels = make_rels_by_clause_joins(root,
 												 old_rel,
 												 other_rels);
-
-			/*
-			 * An exception occurs when there is a clauseless join inside
-			 * an IN (sub-SELECT) construct.  Here, the members of the
-			 * subselect all have join clauses (against the stuff outside
-			 * the IN), but they *must* be joined to each other before we
-			 * can make use of those join clauses.	So do the clauseless
-			 * join bit.
-			 *
-			 * See also the last-ditch case below.
-			 */
-			if (new_rels == NIL && is_inside_IN(root, old_rel))
-				new_rels = make_rels_by_clauseless_joins(root,
-														 old_rel,
-														 other_rels);
 		}
 		else
 		{
 			/*
 			 * Oops, we have a relation that is not joined to any other
-			 * relation.  Cartesian product time.
+			 * relation, either directly or by join-order restrictions.
+			 * Cartesian product time.
 			 */
 			new_rels = make_rels_by_clauseless_joins(root,
 													 old_rel,
@@ -115,36 +102,29 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 		/*
 		 * At levels above 2 we will generate the same joined relation in
 		 * multiple ways --- for example (a join b) join c is the same
-		 * RelOptInfo as (b join c) join a, though the second case will
-		 * add a different set of Paths to it.	To avoid making extra work
-		 * for subsequent passes, do not enter the same RelOptInfo into
-		 * our output list multiple times.
+		 * RelOptInfo as (b join c) join a, though the second case will add a
+		 * different set of Paths to it.  To avoid making extra work for
+		 * subsequent passes, do not enter the same RelOptInfo into our output
+		 * list multiple times.
 		 */
-		foreach(nr, new_rels)
-		{
-			RelOptInfo *jrel = (RelOptInfo *) lfirst(nr);
-
-			if (!list_member_ptr(result_rels, jrel))
-				result_rels = lcons(jrel, result_rels);
-		}
+		result_rels = list_concat_unique_ptr(result_rels, new_rels);
 	}
 
 	/*
-	 * Now, consider "bushy plans" in which relations of k initial rels
-	 * are joined to relations of level-k initial rels, for 2 <= k <=
-	 * level-2.
+	 * Now, consider "bushy plans" in which relations of k initial rels are
+	 * joined to relations of level-k initial rels, for 2 <= k <= level-2.
 	 *
 	 * We only consider bushy-plan joins for pairs of rels where there is a
-	 * suitable join clause, in order to avoid unreasonable growth of
-	 * planning time.
+	 * suitable join clause (or join order restriction), in order to avoid
+	 * unreasonable growth of planning time.
 	 */
 	for (k = 2;; k++)
 	{
 		int			other_level = level - k;
 
 		/*
-		 * Since make_join_rel(x, y) handles both x,y and y,x cases, we
-		 * only need to go as far as the halfway point.
+		 * Since make_join_rel(x, y) handles both x,y and y,x cases, we only
+		 * need to go as far as the halfway point.
 		 */
 		if (k > other_level)
 			break;
@@ -155,8 +135,14 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 			ListCell   *other_rels;
 			ListCell   *r2;
 
-			if (old_rel->joininfo == NIL)
-				continue;		/* we ignore clauseless joins here */
+			/*
+			 * We can ignore clauseless joins here, *except* when they
+			 * participate in join-order restrictions --- then we might have
+			 * to force a bushy join plan.
+			 */
+			if (old_rel->joininfo == NIL &&
+				!has_join_restriction(root, old_rel))
+				continue;
 
 			if (k == other_level)
 				other_rels = lnext(r);	/* only consider remaining rels */
@@ -169,30 +155,22 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 
 				if (!bms_overlap(old_rel->relids, new_rel->relids))
 				{
-					ListCell   *i;
-
 					/*
 					 * OK, we can build a rel of the right level from this
-					 * pair of rels.  Do so if there is at least one
-					 * usable join clause.
+					 * pair of rels.  Do so if there is at least one usable
+					 * join clause or a relevant join restriction.
 					 */
-					foreach(i, old_rel->joininfo)
+					if (have_relevant_joinclause(old_rel, new_rel) ||
+						have_join_order_restriction(root, old_rel, new_rel))
 					{
-						JoinInfo   *joininfo = (JoinInfo *) lfirst(i);
+						RelOptInfo *jrel;
 
-						if (bms_is_subset(joininfo->unjoined_relids,
-										  new_rel->relids))
-						{
-							RelOptInfo *jrel;
-
-							jrel = make_join_rel(root, old_rel, new_rel,
-												 JOIN_INNER);
-							/* Avoid making duplicate entries ... */
-							if (jrel && !list_member_ptr(result_rels, jrel))
-								result_rels = lcons(jrel, result_rels);
-							break;		/* need not consider more
-										 * joininfos */
-						}
+						jrel = make_join_rel(root, old_rel, new_rel,
+											 JOIN_INNER);
+						/* Avoid making duplicate entries ... */
+						if (jrel)
+							result_rels = list_append_unique_ptr(result_rels,
+																 jrel);
 					}
 				}
 			}
@@ -200,16 +178,16 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 	}
 
 	/*
-	 * Last-ditch effort: if we failed to find any usable joins so far,
-	 * force a set of cartesian-product joins to be generated.	This
-	 * handles the special case where all the available rels have join
-	 * clauses but we cannot use any of the joins yet.	An example is
+	 * Last-ditch effort: if we failed to find any usable joins so far, force
+	 * a set of cartesian-product joins to be generated.  This handles the
+	 * special case where all the available rels have join clauses but we
+	 * cannot use any of the joins yet.  An example is
 	 *
 	 * SELECT * FROM a,b,c WHERE (a.f1 + b.f2 + c.f3) = 0;
 	 *
 	 * The join clause will be usable at level 3, but at level 2 we have no
-	 * choice but to make cartesian joins.	We consider only left-sided
-	 * and right-sided cartesian joins in this case (no bushy).
+	 * choice but to make cartesian joins.	We consider only left-sided and
+	 * right-sided cartesian joins in this case (no bushy).
 	 */
 	if (result_rels == NIL)
 	{
@@ -233,13 +211,7 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 													 old_rel,
 													 other_rels);
 
-			foreach(nr, new_rels)
-			{
-				RelOptInfo *jrel = (RelOptInfo *) lfirst(nr);
-
-				if (!list_member_ptr(result_rels, jrel))
-					result_rels = lcons(jrel, result_rels);
-			}
+			result_rels = list_concat_unique_ptr(result_rels, new_rels);
 		}
 
 		/*----------
@@ -269,8 +241,8 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
 /*
  * make_rels_by_clause_joins
  *	  Build joins between the given relation 'old_rel' and other relations
- *	  that are mentioned within old_rel's joininfo nodes (i.e., relations
- *	  that participate in join clauses that 'old_rel' also participates in).
+ *	  that participate in join clauses that 'old_rel' also participates in
+ *	  (or participate in join-order restrictions with it).
  *	  The join rel nodes are returned in a list.
  *
  * 'old_rel' is the relation entry for the relation to be joined
@@ -278,42 +250,29 @@ make_rels_by_joins(Query *root, int level, List **joinrels)
  * rels to be considered for joining
  *
  * Currently, this is only used with initial rels in other_rels, but it
- * will work for joining to joinrels too, if the caller ensures there is no
- * membership overlap between old_rel and the rels in other_rels.  (We need
- * no extra test for overlap for initial rels, since the is_subset test can
- * only succeed when other_rel is not already part of old_rel.)
+ * will work for joining to joinrels too.
  */
 static List *
-make_rels_by_clause_joins(Query *root,
+make_rels_by_clause_joins(PlannerInfo *root,
 						  RelOptInfo *old_rel,
 						  ListCell *other_rels)
 {
 	List	   *result = NIL;
-	ListCell   *i,
-			   *j;
+	ListCell   *l;
 
-	foreach(i, old_rel->joininfo)
+	for_each_cell(l, other_rels)
 	{
-		JoinInfo   *joininfo = (JoinInfo *) lfirst(i);
-		Relids		unjoined_relids = joininfo->unjoined_relids;
+		RelOptInfo *other_rel = (RelOptInfo *) lfirst(l);
 
-		for_each_cell(j, other_rels)
+		if (!bms_overlap(old_rel->relids, other_rel->relids) &&
+			(have_relevant_joinclause(old_rel, other_rel) ||
+			 have_join_order_restriction(root, old_rel, other_rel)))
 		{
-			RelOptInfo *other_rel = (RelOptInfo *) lfirst(j);
+			RelOptInfo *jrel;
 
-			if (bms_is_subset(unjoined_relids, other_rel->relids))
-			{
-				RelOptInfo *jrel;
-
-				jrel = make_join_rel(root, old_rel, other_rel, JOIN_INNER);
-
-				/*
-				 * Avoid entering same joinrel into our output list more
-				 * than once.
-				 */
-				if (jrel && !list_member_ptr(result, jrel))
-					result = lcons(jrel, result);
-			}
+			jrel = make_join_rel(root, old_rel, other_rel, JOIN_INNER);
+			if (jrel)
+				result = lcons(jrel, result);
 		}
 	}
 
@@ -335,7 +294,7 @@ make_rels_by_clause_joins(Query *root,
  * work for joining to joinrels too.
  */
 static List *
-make_rels_by_clauseless_joins(Query *root,
+make_rels_by_clauseless_joins(PlannerInfo *root,
 							  RelOptInfo *old_rel,
 							  ListCell *other_rels)
 {
@@ -353,8 +312,8 @@ make_rels_by_clauseless_joins(Query *root,
 			jrel = make_join_rel(root, old_rel, other_rel, JOIN_INNER);
 
 			/*
-			 * As long as given other_rels are distinct, don't need to
-			 * test to see if jrel is already part of output list.
+			 * As long as given other_rels are distinct, don't need to test to
+			 * see if jrel is already part of output list.
 			 */
 			if (jrel)
 				result = lcons(jrel, result);
@@ -365,29 +324,6 @@ make_rels_by_clauseless_joins(Query *root,
 }
 
 
-/*
- * is_inside_IN
- *		Detect whether the specified relation is inside an IN (sub-SELECT).
- *
- * Note that we are actually only interested in rels that have been pulled up
- * out of an IN, so the routine name is a slight misnomer.
- */
-static bool
-is_inside_IN(Query *root, RelOptInfo *rel)
-{
-	ListCell   *l;
-
-	foreach(l, root->in_info_list)
-	{
-		InClauseInfo *ininfo = (InClauseInfo *) lfirst(l);
-
-		if (bms_is_subset(rel->relids, ininfo->righthand))
-			return true;
-	}
-	return false;
-}
-
-
 /*
  * make_jointree_rel
  *		Find or build a RelOptInfo join rel representing a specific
@@ -395,7 +331,7 @@ is_inside_IN(Query *root, RelOptInfo *rel)
  *		path that corresponds exactly to what the user wrote.
  */
 RelOptInfo *
-make_jointree_rel(Query *root, Node *jtnode)
+make_jointree_rel(PlannerInfo *root, Node *jtnode)
 {
 	if (IsA(jtnode, RangeTblRef))
 	{
@@ -428,10 +364,10 @@ make_jointree_rel(Query *root, Node *jtnode)
 			elog(ERROR, "invalid join order");
 
 		/*
-		 * Since we are only going to consider this one way to do it,
-		 * we're done generating Paths for this joinrel and can now select
-		 * the cheapest.  In fact we *must* do so now, since next level up
-		 * will need it!
+		 * Since we are only going to consider this one way to do it, we're
+		 * done generating Paths for this joinrel and can now select the
+		 * cheapest.  In fact we *must* do so now, since next level up will
+		 * need it!
 		 */
 		set_cheapest(rel);
 
@@ -460,7 +396,7 @@ make_jointree_rel(Query *root, Node *jtnode)
  * happen when working with IN clauses that have been turned into joins.
  */
 RelOptInfo *
-make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
+make_join_rel(PlannerInfo *root, RelOptInfo *rel1, RelOptInfo *rel2,
 			  JoinType jointype)
 {
 	Relids		joinrelids;
@@ -474,10 +410,10 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 	joinrelids = bms_union(rel1->relids, rel2->relids);
 
 	/*
-	 * If we are implementing IN clauses as joins, there are some joins
-	 * that are illegal.  Check to see if the proposed join is trouble. We
-	 * can skip the work if looking at an outer join, however, because
-	 * only top-level joins might be affected.
+	 * If we are implementing IN clauses as joins, there are some joins that
+	 * are illegal.  Check to see if the proposed join is trouble. We can skip
+	 * the work if looking at an outer join, however, because only top-level
+	 * joins might be affected.
 	 */
 	if (jointype == JOIN_INNER)
 	{
@@ -489,8 +425,8 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 
 			/*
 			 * This IN clause is not relevant unless its RHS overlaps the
-			 * proposed join.  (Check this first as a fast path for
-			 * dismissing most irrelevant INs quickly.)
+			 * proposed join.  (Check this first as a fast path for dismissing
+			 * most irrelevant INs quickly.)
 			 */
 			if (!bms_overlap(ininfo->righthand, joinrelids))
 				continue;
@@ -503,10 +439,10 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 				continue;
 
 			/*
-			 * Cannot join if proposed join contains rels not in the RHS
-			 * *and* contains only part of the RHS.  We must build the
-			 * complete RHS (subselect's join) before it can be joined to
-			 * rels outside the subselect.
+			 * Cannot join if proposed join contains rels not in the RHS *and*
+			 * contains only part of the RHS.  We must build the complete RHS
+			 * (subselect's join) before it can be joined to rels outside the
+			 * subselect.
 			 */
 			if (!bms_is_subset(ininfo->righthand, joinrelids))
 			{
@@ -515,13 +451,12 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 			}
 
 			/*
-			 * At this point we are considering a join of the IN's RHS to
-			 * some other rel(s).
+			 * At this point we are considering a join of the IN's RHS to some
+			 * other rel(s).
 			 *
-			 * If we already joined IN's RHS to any other rels in either
-			 * input path, then this join is not constrained (the
-			 * necessary work was done at the lower level where that join
-			 * occurred).
+			 * If we already joined IN's RHS to any other rels in either input
+			 * path, then this join is not constrained (the necessary work was
+			 * done at the lower level where that join occurred).
 			 */
 			if (bms_is_subset(ininfo->righthand, rel1->relids) &&
 				!bms_equal(ininfo->righthand, rel1->relids))
@@ -536,11 +471,11 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 			 * RHS/LHS.
 			 *
 			 * JOIN_UNIQUE_OUTER will work if outerrel is exactly RHS;
-			 * conversely JOIN_UNIQUE_INNER will work if innerrel is
-			 * exactly RHS.
+			 * conversely JOIN_UNIQUE_INNER will work if innerrel is exactly
+			 * RHS.
 			 *
-			 * But none of these will work if we already found another IN
-			 * that needs to trigger here.
+			 * But none of these will work if we already found another IN that
+			 * needs to trigger here.
 			 */
 			if (jointype != JOIN_INNER)
 			{
@@ -567,8 +502,8 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 	}
 
 	/*
-	 * Find or build the join RelOptInfo, and compute the restrictlist
-	 * that goes with this particular joining.
+	 * Find or build the join RelOptInfo, and compute the restrictlist that
+	 * goes with this particular joining.
 	 */
 	joinrel = build_join_rel(root, joinrelids, rel1, rel2, jointype,
 							 &restrictlist);
@@ -642,3 +577,91 @@ make_join_rel(Query *root, RelOptInfo *rel1, RelOptInfo *rel2,
 
 	return joinrel;
 }
+
+
+/*
+ * have_join_order_restriction
+ *		Detect whether the two relations should be joined to satisfy
+ *		a join-order restriction arising from IN clauses.
+ *
+ * In practice this is always used with have_relevant_joinclause(), and so
+ * could be merged with that function, but it seems clearer to separate the
+ * two concerns.  We need these tests because there are degenerate cases where
+ * a clauseless join must be performed to satisfy join-order restrictions.
+ */
+bool
+have_join_order_restriction(PlannerInfo *root,
+							RelOptInfo *rel1, RelOptInfo *rel2)
+{
+	ListCell   *l;
+
+	/*
+	 * It's possible that the rels correspond to the left and right sides
+	 * of a degenerate IN-clause; in which case we should force the join
+	 * to occur.
+	 *
+	 * Also, the two rels could represent a clauseless join that has to be
+	 * completed to build up the LHS or RHS of an IN-clause.
+	 */
+	foreach(l, root->in_info_list)
+	{
+		InClauseInfo *ininfo = (InClauseInfo *) lfirst(l);
+
+		/* Can we perform the IN with these rels? */
+		if (bms_is_subset(ininfo->lefthand, rel1->relids) &&
+			bms_is_subset(ininfo->righthand, rel2->relids))
+			return true;
+		if (bms_is_subset(ininfo->lefthand, rel2->relids) &&
+			bms_is_subset(ininfo->righthand, rel1->relids))
+			return true;
+
+		/*
+		 * Might we need to join these rels to complete the RHS?  It's
+		 * probably overkill to test "overlap", since we never join part of an
+		 * IN's RHS to anything else, but may as well keep the coding similar
+		 * to the OJ case.
+		 */
+		if (bms_overlap(ininfo->righthand, rel1->relids) &&
+			bms_overlap(ininfo->righthand, rel2->relids))
+			return true;
+
+		/* Likewise for the LHS. */
+		if (bms_overlap(ininfo->lefthand, rel1->relids) &&
+			bms_overlap(ininfo->lefthand, rel2->relids))
+			return true;
+	}
+
+	return false;
+}
+
+
+/*
+ * has_join_restriction
+ *		Detect whether the specified relation has join-order restrictions
+ *		due to being inside an IN (sub-SELECT).
+ *
+ * Essentially, this tests whether have_join_order_restriction() could
+ * succeed with this rel and some other one.
+ */
+static bool
+has_join_restriction(PlannerInfo *root, RelOptInfo *rel)
+{
+	ListCell   *l;
+
+	foreach(l, root->in_info_list)
+	{
+		InClauseInfo *ininfo = (InClauseInfo *) lfirst(l);
+
+		/* ignore if IN is already contained in rel */
+		if (bms_is_subset(ininfo->lefthand, rel->relids) &&
+			bms_is_subset(ininfo->righthand, rel->relids))
+			continue;
+
+		/* restricted if it overlaps LHS or RHS, but doesn't contain IN */
+		if (bms_overlap(ininfo->lefthand, rel->relids) ||
+			bms_overlap(ininfo->righthand, rel->relids))
+			return true;
+	}
+
+	return false;
+}
