@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/parser/parse_target.c,v 1.138 2005/10/15 02:49:22 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/parser/parse_target.c,v 1.138.2.2 2006/01/17 17:33:20 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -808,15 +808,16 @@ ExpandIndirectionStar(ParseState *pstate, A_Indirection *ind)
 	 * lookup_rowtype_tupdesc(), which will almost certainly fail as well, but
 	 * it will give an appropriate error message.
 	 *
-	 * If it's a Var of type RECORD, we have to work even harder: we have to find
-	 * what the Var refers to, and pass that to get_expr_result_type. That
-	 * task is handled by expandRecordVariable().
+	 * If it's a Var of type RECORD, we have to work even harder: we have to
+	 * find what the Var refers to, and pass that to get_expr_result_type.
+	 * That task is handled by expandRecordVariable().
 	 */
 	if (IsA(expr, Var) &&
 		((Var *) expr)->vartype == RECORDOID)
 		tupleDesc = expandRecordVariable(pstate, (Var *) expr, 0);
 	else if (get_expr_result_type(expr, NULL, &tupleDesc) != TYPEFUNC_COMPOSITE)
-		tupleDesc = lookup_rowtype_tupdesc(exprType(expr), exprTypmod(expr));
+		tupleDesc = CreateTupleDescCopy(lookup_rowtype_tupdesc(exprType(expr),
+															   exprTypmod(expr)));
 	Assert(tupleDesc);
 
 	/* Generate a list of references to the individual fields */
@@ -993,7 +994,8 @@ expandRecordVariable(ParseState *pstate, Var *var, int levelsup)
 	 * appropriate error message while failing.
 	 */
 	if (get_expr_result_type(expr, NULL, &tupleDesc) != TYPEFUNC_COMPOSITE)
-		tupleDesc = lookup_rowtype_tupdesc(exprType(expr), exprTypmod(expr));
+		tupleDesc = CreateTupleDescCopy(lookup_rowtype_tupdesc(exprType(expr),
+															   exprTypmod(expr)));
 
 	return tupleDesc;
 }
