@@ -4,12 +4,12 @@
  *	  support routines for the lex/flex scanner, used by both the normal
  * backend as well as the bootstrap backend
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql/src/backend/parser/scansup.c,v 1.24.4.1 2004/02/21 00:35:13 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/parser/scansup.c,v 1.30.2.1 2010/05/08 16:40:38 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -53,8 +53,8 @@ scanstr(const char *s)
 		if (s[i] == '\'')
 		{
 			/*
-			 * Note: if scanner is working right, unescaped quotes can
-			 * only appear in pairs, so there should be another character.
+			 * Note: if scanner is working right, unescaped quotes can only
+			 * appear in pairs, so there should be another character.
 			 */
 			i++;
 			newStr[j] = s[i];
@@ -133,6 +133,7 @@ downcase_truncate_identifier(const char *ident, int len, bool warn)
 	int			i;
 
 	result = palloc(len + 1);
+
 	/*
 	 * SQL99 specifies Unicode-aware case normalization, which we don't yet
 	 * have the infrastructure for.  Instead we use tolower() to provide a
@@ -144,7 +145,7 @@ downcase_truncate_identifier(const char *ident, int len, bool warn)
 	 */
 	for (i = 0; i < len; i++)
 	{
-		unsigned char	ch = (unsigned char) ident[i];
+		unsigned char ch = (unsigned char) ident[i];
 
 		if (ch >= 'A' && ch <= 'Z')
 			ch += 'a' - 'A';
@@ -174,12 +175,22 @@ truncate_identifier(char *ident, int len, bool warn)
 {
 	if (len >= NAMEDATALEN)
 	{
-		len = pg_mbcliplen(ident, len, NAMEDATALEN-1);
+		len = pg_mbcliplen(ident, len, NAMEDATALEN - 1);
 		if (warn)
+		{
+			/*
+			 * Cannot use %.*s here because some machines interpret %s's
+			 * precision in characters, others in bytes.
+			 */
+			char	buf[NAMEDATALEN];
+
+			memcpy(buf, ident, len);
+			buf[len] = '\0';
 			ereport(NOTICE,
 					(errcode(ERRCODE_NAME_TOO_LONG),
-					 errmsg("identifier \"%s\" will be truncated to \"%.*s\"",
-							ident, len, ident)));
+					 errmsg("identifier \"%s\" will be truncated to \"%s\"",
+							ident, buf)));
+		}
 		ident[len] = '\0';
 	}
 }
