@@ -8,7 +8,7 @@
  *
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/storage/smgr/md.c,v 1.114.4.2 2007/04/26 23:25:30 tgl Exp $
+ *	  $PostgreSQL: pgsql/src/backend/storage/smgr/md.c,v 1.118.2.4 2007/04/26 23:25:08 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -28,6 +28,26 @@
 #include "utils/memutils.h"
 
 
+/* interval for calling AbsorbFsyncRequests in mdsync */
+#define FSYNCS_PER_ABSORB		10
+
+/* special values for the segno arg to RememberFsyncRequest */
+#define FORGET_RELATION_FSYNC	(InvalidBlockNumber)
+#define FORGET_DATABASE_FSYNC	(InvalidBlockNumber-1)
+
+/*
+ * On Windows, we have to interpret EACCES as possibly meaning the same as
+ * ENOENT, because if a file is unlinked-but-not-yet-gone on that platform,
+ * that's what you get.  Ugh.  This code is designed so that we don't
+ * actually believe these cases are okay without further evidence (namely,
+ * a pending fsync request getting revoked ... see mdsync).
+ */
+#ifndef WIN32
+#define FILE_POSSIBLY_DELETED(err)  ((err) == ENOENT)
+#else
+#define FILE_POSSIBLY_DELETED(err)  ((err) == ENOENT || (err) == EACCES)
+#endif
+
 /*
  *	The magnetic disk storage manager keeps track of open file
  *	descriptors in its own descriptor pool.  This is done to make it
@@ -90,9 +110,8 @@ static MemoryContext MdCxt;		/* context for all md.c allocations */
  * we keep track of pending fsync operations: we need to remember all relation
  * segments that have been written since the last checkpoint, so that we can
  * fsync them down to disk before completing the next checkpoint.  This hash
- * table remembers the pending operations.	We use a hash table not because
- * we want to look up individual operations, but simply as a convenient way
- * of eliminating duplicate requests.
+ * table remembers the pending operations.	We use a hash table mostly as
+ * a convenient way of eliminating duplicate requests.
  *
  * (Regular backends do not track pending operations locally, but forward
  * them to the bgwriter.)
@@ -101,10 +120,21 @@ typedef struct
 {
 	RelFileNode rnode;			/* the targeted relation */
 	BlockNumber segno;			/* which segment */
+} PendingOperationTag;
+
+typedef uint16 CycleCtr;		/* can be any convenient integer size */
+
+typedef struct
+{
+	PendingOperationTag tag;	/* hash table key (must be first!) */
+	bool		canceled;		/* T => request canceled, not yet removed */
+	CycleCtr	cycle_ctr;		/* mdsync_cycle_ctr when request was made */
 } PendingOperationEntry;
 
 static HTAB *pendingOpsTable = NULL;
 
+static CycleCtr mdsync_cycle_ctr = 0;
+
 
 /* local routines */
 static MdfdVec *mdopen(SMgrRelation reln, bool allowNotFound);
@@ -133,9 +163,9 @@ mdinit(void)
 								  ALLOCSET_DEFAULT_MAXSIZE);
 
 	/*
-	 * Create pending-operations hashtable if we need it.  Currently, we
-	 * need it if we are standalone (not under a postmaster) OR if we are
-	 * a bootstrap-mode subprocess of a postmaster (that is, a startup or
+	 * Create pending-operations hashtable if we need it.  Currently, we need
+	 * it if we are standalone (not under a postmaster) OR if we are a
+	 * bootstrap-mode subprocess of a postmaster (that is, a startup or
 	 * bgwriter process).
 	 */
 	if (!IsUnderPostmaster || IsBootstrapProcessingMode())
@@ -143,14 +173,14 @@ mdinit(void)
 		HASHCTL		hash_ctl;
 
 		MemSet(&hash_ctl, 0, sizeof(hash_ctl));
-		hash_ctl.keysize = sizeof(PendingOperationEntry);
+		hash_ctl.keysize = sizeof(PendingOperationTag);
 		hash_ctl.entrysize = sizeof(PendingOperationEntry);
 		hash_ctl.hash = tag_hash;
 		hash_ctl.hcxt = MdCxt;
 		pendingOpsTable = hash_create("Pending Ops Table",
 									  100L,
 									  &hash_ctl,
-							   HASH_ELEM | HASH_FUNCTION | HASH_CONTEXT);
+								   HASH_ELEM | HASH_FUNCTION | HASH_CONTEXT);
 	}
 
 	return true;
@@ -174,21 +204,20 @@ mdcreate(SMgrRelation reln, bool isRedo)
 
 	path = relpath(reln->smgr_rnode);
 
-	fd = FileNameOpenFile(path, O_RDWR | O_CREAT | O_EXCL | PG_BINARY, 0600);
+	fd = PathNameOpenFile(path, O_RDWR | O_CREAT | O_EXCL | PG_BINARY, 0600);
 
 	if (fd < 0)
 	{
 		int			save_errno = errno;
 
 		/*
-		 * During bootstrap, there are cases where a system relation will
-		 * be accessed (by internal backend processes) before the
-		 * bootstrap script nominally creates it.  Therefore, allow the
-		 * file to exist already, even if isRedo is not set.  (See also
-		 * mdopen)
+		 * During bootstrap, there are cases where a system relation will be
+		 * accessed (by internal backend processes) before the bootstrap
+		 * script nominally creates it.  Therefore, allow the file to exist
+		 * already, even if isRedo is not set.	(See also mdopen)
 		 */
 		if (isRedo || IsBootstrapProcessingMode())
-			fd = FileNameOpenFile(path, O_RDWR | PG_BINARY, 0600);
+			fd = PathNameOpenFile(path, O_RDWR | PG_BINARY, 0600);
 		if (fd < 0)
 		{
 			pfree(path);
@@ -227,6 +256,12 @@ mdunlink(RelFileNode rnode, bool isRedo)
 	int			save_errno = 0;
 	char	   *path;
 
+	/*
+	 * We have to clean out any pending fsync requests for the doomed relation,
+	 * else the next mdsync() will fail.
+	 */
+	ForgetRelationFsyncRequests(rnode);
+
 	path = relpath(rnode);
 
 	/* Delete the first segment, or only segment if not doing segmenting */
@@ -301,13 +336,13 @@ mdextend(SMgrRelation reln, BlockNumber blocknum, char *buffer, bool isTemp)
 #endif
 
 	/*
-	 * Note: because caller obtained blocknum by calling _mdnblocks, which
-	 * did a seek(SEEK_END), this seek is often redundant and will be
-	 * optimized away by fd.c.	It's not redundant, however, if there is a
-	 * partial page at the end of the file.  In that case we want to try
-	 * to overwrite the partial page with a full page.	It's also not
-	 * redundant if bufmgr.c had to dump another buffer of the same file
-	 * to make room for the new page's buffer.
+	 * Note: because caller obtained blocknum by calling _mdnblocks, which did
+	 * a seek(SEEK_END), this seek is often redundant and will be optimized
+	 * away by fd.c.  It's not redundant, however, if there is a partial page
+	 * at the end of the file.	In that case we want to try to overwrite the
+	 * partial page with a full page.  It's also not redundant if bufmgr.c had
+	 * to dump another buffer of the same file to make room for the new page's
+	 * buffer.
 	 */
 	if (FileSeek(v->mdfd_vfd, seekpos, SEEK_SET) != seekpos)
 		return false;
@@ -358,23 +393,22 @@ mdopen(SMgrRelation reln, bool allowNotFound)
 
 	path = relpath(reln->smgr_rnode);
 
-	fd = FileNameOpenFile(path, O_RDWR | PG_BINARY, 0600);
+	fd = PathNameOpenFile(path, O_RDWR | PG_BINARY, 0600);
 
 	if (fd < 0)
 	{
 		/*
-		 * During bootstrap, there are cases where a system relation will
-		 * be accessed (by internal backend processes) before the
-		 * bootstrap script nominally creates it.  Therefore, accept
-		 * mdopen() as a substitute for mdcreate() in bootstrap mode only.
-		 * (See mdcreate)
+		 * During bootstrap, there are cases where a system relation will be
+		 * accessed (by internal backend processes) before the bootstrap
+		 * script nominally creates it.  Therefore, accept mdopen() as a
+		 * substitute for mdcreate() in bootstrap mode only. (See mdcreate)
 		 */
 		if (IsBootstrapProcessingMode())
-			fd = FileNameOpenFile(path, O_RDWR | O_CREAT | O_EXCL | PG_BINARY, 0600);
+			fd = PathNameOpenFile(path, O_RDWR | O_CREAT | O_EXCL | PG_BINARY, 0600);
 		if (fd < 0)
 		{
 			pfree(path);
-			if (allowNotFound && errno == ENOENT)
+			if (allowNotFound && FILE_POSSIBLY_DELETED(errno))
 				return NULL;
 			ereport(ERROR,
 					(errcode_for_file_access(),
@@ -463,8 +497,8 @@ mdread(SMgrRelation reln, BlockNumber blocknum, char *buffer)
 	if ((nbytes = FileRead(v->mdfd_vfd, buffer, BLCKSZ)) != BLCKSZ)
 	{
 		/*
-		 * If we are at or past EOF, return zeroes without complaining.
-		 * Also substitute zeroes if we found a partial block at EOF.
+		 * If we are at or past EOF, return zeroes without complaining. Also
+		 * substitute zeroes if we found a partial block at EOF.
 		 *
 		 * XXX this is really ugly, bad design.  However the current
 		 * implementation of hash indexes requires it, because hash index
@@ -533,10 +567,9 @@ mdnblocks(SMgrRelation reln)
 	BlockNumber segno = 0;
 
 	/*
-	 * Skip through any segments that aren't the last one, to avoid
-	 * redundant seeks on them.  We have previously verified that these
-	 * segments are exactly RELSEG_SIZE long, and it's useless to recheck
-	 * that each time.
+	 * Skip through any segments that aren't the last one, to avoid redundant
+	 * seeks on them.  We have previously verified that these segments are
+	 * exactly RELSEG_SIZE long, and it's useless to recheck that each time.
 	 *
 	 * NOTE: this assumption could only be wrong if another backend has
 	 * truncated the relation.	We rely on higher code levels to handle that
@@ -567,11 +600,10 @@ mdnblocks(SMgrRelation reln)
 		if (v->mdfd_chain == NULL)
 		{
 			/*
-			 * Because we pass O_CREAT, we will create the next segment
-			 * (with zero length) immediately, if the last segment is of
-			 * length REL_SEGSIZE.	This is unnecessary but harmless, and
-			 * testing for the case would take more cycles than it seems
-			 * worth.
+			 * Because we pass O_CREAT, we will create the next segment (with
+			 * zero length) immediately, if the last segment is of length
+			 * RELSEG_SIZE.  While perhaps not strictly necessary, this keeps
+			 * the logic simple.
 			 */
 			v->mdfd_chain = _mdfd_openseg(reln, segno, O_CREAT);
 			if (v->mdfd_chain == NULL)
@@ -641,12 +673,12 @@ mdtruncate(SMgrRelation reln, BlockNumber nblocks, bool isTemp)
 		else if (priorblocks + ((BlockNumber) RELSEG_SIZE) > nblocks)
 		{
 			/*
-			 * This is the last segment we want to keep. Truncate the file
-			 * to the right length, and clear chain link that points to
-			 * any remaining segments (which we shall zap). NOTE: if
-			 * nblocks is exactly a multiple K of RELSEG_SIZE, we will
-			 * truncate the K+1st segment to 0 length but keep it. This is
-			 * mainly so that the right thing happens if nblocks==0.
+			 * This is the last segment we want to keep. Truncate the file to
+			 * the right length, and clear chain link that points to any
+			 * remaining segments (which we shall zap). NOTE: if nblocks is
+			 * exactly a multiple K of RELSEG_SIZE, we will truncate the K+1st
+			 * segment to 0 length but keep it. This adheres to the invariant
+			 * given in the header comments.
 			 */
 			BlockNumber lastsegblocks = nblocks - priorblocks;
 
@@ -663,8 +695,8 @@ mdtruncate(SMgrRelation reln, BlockNumber nblocks, bool isTemp)
 		else
 		{
 			/*
-			 * We still need this segment and 0 or more blocks beyond it,
-			 * so nothing to do here.
+			 * We still need this segment and 0 or more blocks beyond it, so
+			 * nothing to do here.
 			 */
 			v = v->mdfd_chain;
 		}
@@ -685,6 +717,9 @@ mdtruncate(SMgrRelation reln, BlockNumber nblocks, bool isTemp)
 
 /*
  *	mdimmedsync() -- Immediately sync a relation to stable storage.
+ *
+ * Note that only writes already issued are synced; this routine knows
+ * nothing of dirty buffers that may exist inside the buffer manager.
  */
 bool
 mdimmedsync(SMgrRelation reln)
@@ -719,16 +754,20 @@ mdimmedsync(SMgrRelation reln)
 
 /*
  *	mdsync() -- Sync previous writes to stable storage.
- *
- * This is only called during checkpoints, and checkpoints should only
- * occur in processes that have created a pendingOpsTable.
  */
 bool
 mdsync(void)
 {
+	static bool mdsync_in_progress = false;
+
 	HASH_SEQ_STATUS hstat;
 	PendingOperationEntry *entry;
+	int			absorb_counter;
 
+	/*
+	 * This is only called during checkpoints, and checkpoints should only
+	 * occur in processes that have created a pendingOpsTable.
+	 */
 	if (!pendingOpsTable)
 		return false;
 
@@ -740,69 +779,182 @@ mdsync(void)
 	 */
 	AbsorbFsyncRequests();
 
+	/*
+	 * To avoid excess fsync'ing (in the worst case, maybe a never-terminating
+	 * checkpoint), we want to ignore fsync requests that are entered into the
+	 * hashtable after this point --- they should be processed next time,
+	 * instead.  We use mdsync_cycle_ctr to tell old entries apart from new
+	 * ones: new ones will have cycle_ctr equal to the incremented value of
+	 * mdsync_cycle_ctr.
+	 *
+	 * In normal circumstances, all entries present in the table at this
+	 * point will have cycle_ctr exactly equal to the current (about to be old)
+	 * value of mdsync_cycle_ctr.  However, if we fail partway through the
+	 * fsync'ing loop, then older values of cycle_ctr might remain when we
+	 * come back here to try again.  Repeated checkpoint failures would
+	 * eventually wrap the counter around to the point where an old entry
+	 * might appear new, causing us to skip it, possibly allowing a checkpoint
+	 * to succeed that should not have.  To forestall wraparound, any time
+	 * the previous mdsync() failed to complete, run through the table and
+	 * forcibly set cycle_ctr = mdsync_cycle_ctr.
+	 *
+	 * Think not to merge this loop with the main loop, as the problem is
+	 * exactly that that loop may fail before having visited all the entries.
+	 * From a performance point of view it doesn't matter anyway, as this
+	 * path will never be taken in a system that's functioning normally.
+	 */
+	if (mdsync_in_progress)
+	{
+		/* prior try failed, so update any stale cycle_ctr values */
+		hash_seq_init(&hstat, pendingOpsTable);
+		while ((entry = (PendingOperationEntry *) hash_seq_search(&hstat)) != NULL)
+		{
+			entry->cycle_ctr = mdsync_cycle_ctr;
+		}
+	}
+
+	/* Advance counter so that new hashtable entries are distinguishable */
+	mdsync_cycle_ctr++;
+
+	/* Set flag to detect failure if we don't reach the end of the loop */
+	mdsync_in_progress = true;
+
+	/* Now scan the hashtable for fsync requests to process */
+	absorb_counter = FSYNCS_PER_ABSORB;
 	hash_seq_init(&hstat, pendingOpsTable);
 	while ((entry = (PendingOperationEntry *) hash_seq_search(&hstat)) != NULL)
 	{
 		/*
-		 * If fsync is off then we don't have to bother opening the file
-		 * at all.	(We delay checking until this point so that changing
-		 * fsync on the fly behaves sensibly.)
+		 * If the entry is new then don't process it this time.  Note that
+		 * "continue" bypasses the hash-remove call at the bottom of the loop.
 		 */
-		if (enableFsync)
+		if (entry->cycle_ctr == mdsync_cycle_ctr)
+			continue;
+
+		/* Else assert we haven't missed it */
+		Assert((CycleCtr) (entry->cycle_ctr + 1) == mdsync_cycle_ctr);
+
+		/*
+		 * If fsync is off then we don't have to bother opening the file
+		 * at all.  (We delay checking until this point so that changing
+		 * fsync on the fly behaves sensibly.)  Also, if the entry is
+		 * marked canceled, fall through to delete it.
+		 */
+		if (enableFsync && !entry->canceled)
 		{
-			SMgrRelation reln;
-			MdfdVec    *seg;
+			int			failures;
 
 			/*
-			 * Find or create an smgr hash entry for this relation. This
-			 * may seem a bit unclean -- md calling smgr?  But it's really
-			 * the best solution.  It ensures that the open file reference
-			 * isn't permanently leaked if we get an error here. (You may
-			 * say "but an unreferenced SMgrRelation is still a leak!" Not
-			 * really, because the only case in which a checkpoint is done
-			 * by a process that isn't about to shut down is in the
-			 * bgwriter, and it will periodically do smgrcloseall().  This
-			 * fact justifies our not closing the reln in the success path
-			 * either, which is a good thing since in non-bgwriter cases
-			 * we couldn't safely do that.)  Furthermore, in many cases
-			 * the relation will have been dirtied through this same smgr
-			 * relation, and so we can save a file open/close cycle.
+			 * If in bgwriter, we want to absorb pending requests every so
+			 * often to prevent overflow of the fsync request queue.  It is
+			 * unspecified whether newly-added entries will be visited by
+			 * hash_seq_search, but we don't care since we don't need to
+			 * process them anyway.
 			 */
-			reln = smgropen(entry->rnode);
-
-			/*
-			 * It is possible that the relation has been dropped or
-			 * truncated since the fsync request was entered.  Therefore,
-			 * we have to allow file-not-found errors.	This applies both
-			 * during _mdfd_getseg() and during FileSync, since fd.c might
-			 * have closed the file behind our back.
-			 */
-			seg = _mdfd_getseg(reln,
-							   entry->segno * ((BlockNumber) RELSEG_SIZE),
-							   true);
-			if (seg)
+			if (--absorb_counter <= 0)
 			{
-				if (FileSync(seg->mdfd_vfd) < 0 &&
-					errno != ENOENT)
+				AbsorbFsyncRequests();
+				absorb_counter = FSYNCS_PER_ABSORB;
+			}
+
+			/*
+			 * The fsync table could contain requests to fsync segments that
+			 * have been deleted (unlinked) by the time we get to them.
+			 * Rather than just hoping an ENOENT (or EACCES on Windows) error
+			 * can be ignored, what we do on error is absorb pending requests
+			 * and then retry.  Since mdunlink() queues a "revoke" message
+			 * before actually unlinking, the fsync request is guaranteed to
+			 * be marked canceled after the absorb if it really was this case.
+			 * DROP DATABASE likewise has to tell us to forget fsync requests
+			 * before it starts deletions.
+			 */
+			for (failures = 0; ; failures++)	/* loop exits at "break" */
+			{
+				SMgrRelation reln;
+				MdfdVec    *seg;
+
+				/*
+				 * Find or create an smgr hash entry for this relation. This
+				 * may seem a bit unclean -- md calling smgr?  But it's really
+				 * the best solution.  It ensures that the open file reference
+				 * isn't permanently leaked if we get an error here. (You may
+				 * say "but an unreferenced SMgrRelation is still a leak!" Not
+				 * really, because the only case in which a checkpoint is done
+				 * by a process that isn't about to shut down is in the
+				 * bgwriter, and it will periodically do smgrcloseall(). This
+				 * fact justifies our not closing the reln in the success path
+				 * either, which is a good thing since in non-bgwriter cases
+				 * we couldn't safely do that.)  Furthermore, in many cases
+				 * the relation will have been dirtied through this same smgr
+				 * relation, and so we can save a file open/close cycle.
+				 */
+				reln = smgropen(entry->tag.rnode);
+
+				/*
+				 * It is possible that the relation has been dropped or
+				 * truncated since the fsync request was entered.  Therefore,
+				 * allow ENOENT, but only if we didn't fail already on
+				 * this file.  This applies both during _mdfd_getseg() and
+				 * during FileSync, since fd.c might have closed the file
+				 * behind our back.
+				 */
+				seg = _mdfd_getseg(reln,
+								   entry->tag.segno * ((BlockNumber) RELSEG_SIZE),
+								   true);
+				if (seg != NULL &&
+					FileSync(seg->mdfd_vfd) >= 0)
+					break;		/* success; break out of retry loop */
+
+				/*
+				 * XXX is there any point in allowing more than one retry?
+				 * Don't see one at the moment, but easy to change the
+				 * test here if so.
+				 */
+				if (!FILE_POSSIBLY_DELETED(errno) ||
+					failures > 0)
 				{
 					ereport(LOG,
 							(errcode_for_file_access(),
 							 errmsg("could not fsync segment %u of relation %u/%u/%u: %m",
-									entry->segno,
-									entry->rnode.spcNode,
-									entry->rnode.dbNode,
-									entry->rnode.relNode)));
+									entry->tag.segno,
+									entry->tag.rnode.spcNode,
+									entry->tag.rnode.dbNode,
+									entry->tag.rnode.relNode)));
 					hash_seq_term(&hstat);
 					return false;
 				}
-			}
+				else
+					ereport(DEBUG1,
+							(errcode_for_file_access(),
+							 errmsg("could not fsync segment %u of relation %u/%u/%u, but retrying: %m",
+									entry->tag.segno,
+									entry->tag.rnode.spcNode,
+									entry->tag.rnode.dbNode,
+									entry->tag.rnode.relNode)));
+
+				/*
+				 * Absorb incoming requests and check to see if canceled.
+				 */
+				AbsorbFsyncRequests();
+				absorb_counter = FSYNCS_PER_ABSORB;	/* might as well... */
+
+				if (entry->canceled)
+					break;
+			}	/* end retry loop */
 		}
 
-		/* Okay, delete this entry */
-		if (hash_search(pendingOpsTable, entry,
+		/*
+		 * If we get here, either we fsync'd successfully, or we don't have
+		 * to because enableFsync is off, or the entry is (now) marked
+		 * canceled.  Okay to delete it.
+		 */
+		if (hash_search(pendingOpsTable, &entry->tag,
 						HASH_REMOVE, NULL) == NULL)
 			elog(ERROR, "pendingOpsTable corrupted");
-	}
+	}	/* end loop over hashtable entries */
+
+	/* Flag successful completion of mdsync */
+	mdsync_in_progress = false;
 
 	return true;
 }
@@ -824,16 +976,9 @@ register_dirty_segment(SMgrRelation reln, MdfdVec *seg)
 {
 	if (pendingOpsTable)
 	{
-		PendingOperationEntry entry;
-
-		/* ensure any pad bytes in the struct are zeroed */
-		MemSet(&entry, 0, sizeof(entry));
-		entry.rnode = reln->smgr_rnode;
-		entry.segno = seg->mdfd_segno;
-
-		if (hash_search(pendingOpsTable, &entry, HASH_ENTER, NULL) != NULL)
-			return true;
-		/* out of memory: fall through to do it locally */
+		/* push it into local pending-ops table */
+		RememberFsyncRequest(reln->smgr_rnode, seg->mdfd_segno);
+		return true;
 	}
 	else
 	{
@@ -851,25 +996,148 @@ register_dirty_segment(SMgrRelation reln, MdfdVec *seg)
  *
  * We stuff the fsync request into the local hash table for execution
  * during the bgwriter's next checkpoint.
+ *
+ * The range of possible segment numbers is way less than the range of
+ * BlockNumber, so we can reserve high values of segno for special purposes.
+ * We define two: FORGET_RELATION_FSYNC means to cancel pending fsyncs for
+ * a relation, and FORGET_DATABASE_FSYNC means to cancel pending fsyncs for
+ * a whole database.  (These are a tad slow because the hash table has to be
+ * searched linearly, but it doesn't seem worth rethinking the table structure
+ * for them.)
  */
 void
 RememberFsyncRequest(RelFileNode rnode, BlockNumber segno)
 {
-	PendingOperationEntry entry;
-
 	Assert(pendingOpsTable);
 
-	/* ensure any pad bytes in the struct are zeroed */
-	MemSet(&entry, 0, sizeof(entry));
-	entry.rnode = rnode;
-	entry.segno = segno;
+	if (segno == FORGET_RELATION_FSYNC)
+	{
+		/* Remove any pending requests for the entire relation */
+		HASH_SEQ_STATUS hstat;
+		PendingOperationEntry *entry;
 
-	if (hash_search(pendingOpsTable, &entry, HASH_ENTER, NULL) == NULL)
-		ereport(FATAL,
-				(errcode(ERRCODE_OUT_OF_MEMORY),
-				 errmsg("out of memory")));
+		hash_seq_init(&hstat, pendingOpsTable);
+		while ((entry = (PendingOperationEntry *) hash_seq_search(&hstat)) != NULL)
+		{
+			if (RelFileNodeEquals(entry->tag.rnode, rnode))
+			{
+				/* Okay, cancel this entry */
+				entry->canceled = true;
+			}
+		}
+	}
+	else if (segno == FORGET_DATABASE_FSYNC)
+	{
+		/* Remove any pending requests for the entire database */
+		HASH_SEQ_STATUS hstat;
+		PendingOperationEntry *entry;
+
+		hash_seq_init(&hstat, pendingOpsTable);
+		while ((entry = (PendingOperationEntry *) hash_seq_search(&hstat)) != NULL)
+		{
+			if (entry->tag.rnode.dbNode == rnode.dbNode)
+			{
+				/* Okay, cancel this entry */
+				entry->canceled = true;
+			}
+		}
+	}
+	else
+	{
+		/* Normal case: enter a request to fsync this segment */
+		PendingOperationTag key;
+		PendingOperationEntry *entry;
+		bool		found;
+
+		/* ensure any pad bytes in the hash key are zeroed */
+		MemSet(&key, 0, sizeof(key));
+		key.rnode = rnode;
+		key.segno = segno;
+
+		entry = (PendingOperationEntry *) hash_search(pendingOpsTable,
+													  &key,
+													  HASH_ENTER,
+													  &found);
+		/* if new or previously canceled entry, initialize it */
+		if (!found || entry->canceled)
+		{
+			entry->canceled = false;
+			entry->cycle_ctr = mdsync_cycle_ctr;
+		}
+		/*
+		 * NB: it's intentional that we don't change cycle_ctr if the entry
+		 * already exists.  The fsync request must be treated as old, even
+		 * though the new request will be satisfied too by any subsequent
+		 * fsync.
+		 *
+		 * However, if the entry is present but is marked canceled, we should
+		 * act just as though it wasn't there.  The only case where this could
+		 * happen would be if a file had been deleted, we received but did not
+		 * yet act on the cancel request, and the same relfilenode was then
+		 * assigned to a new file.  We mustn't lose the new request, but
+		 * it should be considered new not old.
+		 */
+	}
 }
 
+/*
+ * ForgetRelationFsyncRequests -- ensure any fsyncs for a rel are forgotten
+ */
+void
+ForgetRelationFsyncRequests(RelFileNode rnode)
+{
+	if (pendingOpsTable)
+	{
+		/* standalone backend or startup process: fsync state is local */
+		RememberFsyncRequest(rnode, FORGET_RELATION_FSYNC);
+	}
+	else if (IsUnderPostmaster)
+	{
+		/*
+		 * Notify the bgwriter about it.  If we fail to queue the revoke
+		 * message, we have to sleep and try again ... ugly, but hopefully
+		 * won't happen often.
+		 *
+		 * XXX should we CHECK_FOR_INTERRUPTS in this loop?  Escaping with
+		 * an error would leave the no-longer-used file still present on
+		 * disk, which would be bad, so I'm inclined to assume that the
+		 * bgwriter will always empty the queue soon.
+		 */
+		while (!ForwardFsyncRequest(rnode, FORGET_RELATION_FSYNC))
+			pg_usleep(10000L);	/* 10 msec seems a good number */
+		/*
+		 * Note we don't wait for the bgwriter to actually absorb the
+		 * revoke message; see mdsync() for the implications.
+		 */
+	}
+}
+
+/*
+ * ForgetDatabaseFsyncRequests -- ensure any fsyncs for a DB are forgotten
+ */
+void
+ForgetDatabaseFsyncRequests(Oid dbid)
+{
+	RelFileNode rnode;
+
+	rnode.dbNode = dbid;
+	rnode.spcNode = 0;
+	rnode.relNode = 0;
+
+	if (pendingOpsTable)
+	{
+		/* standalone backend or startup process: fsync state is local */
+		RememberFsyncRequest(rnode, FORGET_DATABASE_FSYNC);
+	}
+	else if (IsUnderPostmaster)
+	{
+		/* see notes in ForgetRelationFsyncRequests */
+		while (!ForwardFsyncRequest(rnode, FORGET_DATABASE_FSYNC))
+			pg_usleep(10000L);	/* 10 msec seems a good number */
+	}
+}
+
+
 /*
  *	_fdvec_alloc() -- Make a MdfdVec object.
  */
@@ -906,7 +1174,7 @@ _mdfd_openseg(SMgrRelation reln, BlockNumber segno, int oflags)
 		fullpath = path;
 
 	/* open the file */
-	fd = FileNameOpenFile(fullpath, O_RDWR | PG_BINARY | oflags, 0600);
+	fd = PathNameOpenFile(fullpath, O_RDWR | PG_BINARY | oflags, 0600);
 
 	pfree(fullpath);
 
@@ -952,29 +1220,28 @@ _mdfd_getseg(SMgrRelation reln, BlockNumber blkno, bool allowNotFound)
 		{
 			/*
 			 * We will create the next segment only if the target block is
-			 * within it.  This prevents Sorcerer's Apprentice syndrome if
-			 * a bug at higher levels causes us to be handed a
-			 * ridiculously large blkno --- otherwise we could create many
-			 * thousands of empty segment files before reaching the
-			 * "target" block.	We should never need to create more than
-			 * one new segment per call, so this restriction seems
-			 * reasonable.
+			 * within it.  This prevents Sorcerer's Apprentice syndrome if a
+			 * bug at higher levels causes us to be handed a ridiculously
+			 * large blkno --- otherwise we could create many thousands of
+			 * empty segment files before reaching the "target" block.	We
+			 * should never need to create more than one new segment per call,
+			 * so this restriction seems reasonable.
 			 *
 			 * BUT: when doing WAL recovery, disable this logic and create
-			 * segments unconditionally.  In this case it seems better
-			 * to assume the given blkno is good (it presumably came from
-			 * a CRC-checked WAL record); furthermore this lets us cope
-			 * in the case where we are replaying WAL data that has a write
-			 * into a high-numbered segment of a relation that was later
-			 * deleted.  We want to go ahead and create the segments so
-			 * we can finish out the replay.
+			 * segments unconditionally.  In this case it seems better to
+			 * assume the given blkno is good (it presumably came from a
+			 * CRC-checked WAL record); furthermore this lets us cope in the
+			 * case where we are replaying WAL data that has a write into a
+			 * high-numbered segment of a relation that was later deleted.	We
+			 * want to go ahead and create the segments so we can finish out
+			 * the replay.
 			 */
 			v->mdfd_chain = _mdfd_openseg(reln,
 										  nextsegno,
-								  (segstogo == 1 || InRecovery) ? O_CREAT : 0);
+								(segstogo == 1 || InRecovery) ? O_CREAT : 0);
 			if (v->mdfd_chain == NULL)
 			{
-				if (allowNotFound && errno == ENOENT)
+				if (allowNotFound && FILE_POSSIBLY_DELETED(errno))
 					return NULL;
 				ereport(ERROR,
 						(errcode_for_file_access(),
