@@ -3,12 +3,12 @@
  * int.c
  *	  Functions for the built-in integer types (except int8).
  *
- * Portions Copyright (c) 1996-2003, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2005, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/utils/adt/int.c,v 1.57 2003/08/04 02:40:05 momjian Exp $
+ *	  $PostgreSQL: pgsql/src/backend/utils/adt/int.c,v 1.68.2.3 2006/06/12 16:29:08 momjian Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -26,17 +26,20 @@
  *		 intpl, intmi, int4mul, intdiv
  *
  *		Arithmetic operators:
- *		 intmod, int4fac
+ *		 intmod
  */
-
 #include "postgres.h"
 
 #include <ctype.h>
 #include <limits.h>
 
+#include "catalog/pg_type.h"
+#include "funcapi.h"
 #include "libpq/pqformat.h"
+#include "utils/array.h"
 #include "utils/builtins.h"
 
+
 #ifndef SHRT_MAX
 #define SHRT_MAX (0x7FFF)
 #endif
@@ -44,6 +47,18 @@
 #define SHRT_MIN (-0x8000)
 #endif
 
+#define SAMESIGN(a,b)	(((a) < 0) == ((b) < 0))
+
+#define Int2VectorSize(n)	(offsetof(int2vector, values) + (n) * sizeof(int2))
+
+typedef struct
+{
+	int32		current;
+	int32		finish;
+	int32		step;
+} generate_series_fctx;
+
+
 /*****************************************************************************
  *	 USER I/O ROUTINES														 *
  *****************************************************************************/
@@ -98,20 +113,49 @@ int2send(PG_FUNCTION_ARGS)
 }
 
 /*
- *		int2vectorin			- converts "num num ..." to internal form
+ * construct int2vector given a raw array of int2s
  *
- *		Note: Fills any missing slots with zeroes.
+ * If int2s is NULL then caller must fill values[] afterward
+ */
+int2vector *
+buildint2vector(const int2 *int2s, int n)
+{
+	int2vector *result;
+
+	result = (int2vector *) palloc0(Int2VectorSize(n));
+
+	if (n > 0 && int2s)
+		memcpy(result->values, int2s, n * sizeof(int2));
+
+	/*
+	 * Attach standard array header.  For historical reasons, we set the index
+	 * lower bound to 0 not 1.
+	 */
+	result->size = Int2VectorSize(n);
+	result->ndim = 1;
+	result->flags = 0;
+	result->elemtype = INT2OID;
+	result->dim1 = n;
+	result->lbound1 = 0;
+
+	return result;
+}
+
+/*
+ *		int2vectorin			- converts "num num ..." to internal form
  */
 Datum
 int2vectorin(PG_FUNCTION_ARGS)
 {
 	char	   *intString = PG_GETARG_CSTRING(0);
-	int16	   *result = (int16 *) palloc(sizeof(int16[INDEX_MAX_KEYS]));
-	int			slot;
+	int2vector *result;
+	int			n;
 
-	for (slot = 0; *intString && slot < INDEX_MAX_KEYS; slot++)
+	result = (int2vector *) palloc0(Int2VectorSize(FUNC_MAX_ARGS));
+
+	for (n = 0; *intString && n < FUNC_MAX_ARGS; n++)
 	{
-		if (sscanf(intString, "%hd", &result[slot]) != 1)
+		if (sscanf(intString, "%hd", &result->values[n]) != 1)
 			break;
 		while (*intString && isspace((unsigned char) *intString))
 			intString++;
@@ -125,8 +169,12 @@ int2vectorin(PG_FUNCTION_ARGS)
 				(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
 				 errmsg("int2vector has too many elements")));
 
-	while (slot < INDEX_MAX_KEYS)
-		result[slot++] = 0;
+	result->size = Int2VectorSize(n);
+	result->ndim = 1;
+	result->flags = 0;
+	result->elemtype = INT2OID;
+	result->dim1 = n;
+	result->lbound1 = 0;
 
 	PG_RETURN_POINTER(result);
 }
@@ -137,24 +185,19 @@ int2vectorin(PG_FUNCTION_ARGS)
 Datum
 int2vectorout(PG_FUNCTION_ARGS)
 {
-	int16	   *int2Array = (int16 *) PG_GETARG_POINTER(0);
+	int2vector *int2Array = (int2vector *) PG_GETARG_POINTER(0);
 	int			num,
-				maxnum;
+				nnums = int2Array->dim1;
 	char	   *rp;
 	char	   *result;
 
-	/* find last non-zero value in vector */
-	for (maxnum = INDEX_MAX_KEYS - 1; maxnum >= 0; maxnum--)
-		if (int2Array[maxnum] != 0)
-			break;
-
 	/* assumes sign, 5 digits, ' ' */
-	rp = result = (char *) palloc((maxnum + 1) * 7 + 1);
-	for (num = 0; num <= maxnum; num++)
+	rp = result = (char *) palloc(nnums * 7 + 1);
+	for (num = 0; num < nnums; num++)
 	{
 		if (num != 0)
 			*rp++ = ' ';
-		pg_itoa(int2Array[num], rp);
+		pg_itoa(int2Array->values[num], rp);
 		while (*++rp != '\0')
 			;
 	}
@@ -169,11 +212,35 @@ Datum
 int2vectorrecv(PG_FUNCTION_ARGS)
 {
 	StringInfo	buf = (StringInfo) PG_GETARG_POINTER(0);
-	int16	   *result = (int16 *) palloc(sizeof(int16[INDEX_MAX_KEYS]));
-	int			slot;
+	FunctionCallInfoData locfcinfo;
+	int2vector *result;
 
-	for (slot = 0; slot < INDEX_MAX_KEYS; slot++)
-		result[slot] = (int16) pq_getmsgint(buf, sizeof(int16));
+	/*
+	 * Normally one would call array_recv() using DirectFunctionCall3,
+	 * but that does not work since array_recv wants to cache some data
+	 * using fcinfo->flinfo->fn_extra.  So we need to pass it our own
+	 * flinfo parameter.
+	 */
+	InitFunctionCallInfoData(locfcinfo, fcinfo->flinfo, 3, NULL, NULL);
+
+	locfcinfo.arg[0] = PointerGetDatum(buf);
+	locfcinfo.arg[1] = ObjectIdGetDatum(INT2OID);
+	locfcinfo.arg[2] = Int32GetDatum(-1);
+	locfcinfo.argnull[0] = false;
+	locfcinfo.argnull[1] = false;
+	locfcinfo.argnull[2] = false;
+
+	result = (int2vector *) DatumGetPointer(array_recv(&locfcinfo));
+
+	Assert(!locfcinfo.isnull);
+
+	/* sanity checks: int2vector must be 1-D, no nulls */
+	if (result->ndim != 1 ||
+		result->flags != 0 ||
+		result->elemtype != INT2OID)
+		ereport(ERROR,
+				(errcode(ERRCODE_INVALID_BINARY_REPRESENTATION),
+				 errmsg("invalid int2vector data")));
 	PG_RETURN_POINTER(result);
 }
 
@@ -183,14 +250,7 @@ int2vectorrecv(PG_FUNCTION_ARGS)
 Datum
 int2vectorsend(PG_FUNCTION_ARGS)
 {
-	int16	   *int2Array = (int16 *) PG_GETARG_POINTER(0);
-	StringInfoData buf;
-	int			slot;
-
-	pq_begintypsend(&buf);
-	for (slot = 0; slot < INDEX_MAX_KEYS; slot++)
-		pq_sendint(&buf, int2Array[slot], sizeof(int16));
-	PG_RETURN_BYTEA_P(pq_endtypsend(&buf));
+	return array_send(fcinfo);
 }
 
 /*
@@ -200,10 +260,12 @@ int2vectorsend(PG_FUNCTION_ARGS)
 Datum
 int2vectoreq(PG_FUNCTION_ARGS)
 {
-	int16	   *arg1 = (int16 *) PG_GETARG_POINTER(0);
-	int16	   *arg2 = (int16 *) PG_GETARG_POINTER(1);
+	int2vector *a = (int2vector *) PG_GETARG_POINTER(0);
+	int2vector *b = (int2vector *) PG_GETARG_POINTER(1);
 
-	PG_RETURN_BOOL(memcmp(arg1, arg2, INDEX_MAX_KEYS * sizeof(int16)) == 0);
+	if (a->dim1 != b->dim1)
+		PG_RETURN_BOOL(false);
+	PG_RETURN_BOOL(memcmp(a->values, b->values, a->dim1 * sizeof(int2)) == 0);
 }
 
 
@@ -283,7 +345,7 @@ i4toi2(PG_FUNCTION_ARGS)
 	if (arg1 < SHRT_MIN || arg1 > SHRT_MAX)
 		ereport(ERROR,
 				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
-				 errmsg("integer out of range")));
+				 errmsg("smallint out of range")));
 
 	PG_RETURN_INT16((int16) arg1);
 }
@@ -350,6 +412,25 @@ text_int4(PG_FUNCTION_ARGS)
 	return result;
 }
 
+/* Cast int4 -> bool */
+Datum
+int4_bool(PG_FUNCTION_ARGS)
+{
+	if (PG_GETARG_INT32(0) == 0)
+		PG_RETURN_BOOL(false);
+	else
+		PG_RETURN_BOOL(true);
+}
+
+/* Cast bool -> int4 */
+Datum
+bool_int4(PG_FUNCTION_ARGS)
+{
+	if (PG_GETARG_BOOL(0) == false)
+		PG_RETURN_INT32(0);
+	else
+		PG_RETURN_INT32(1);
+}
 
 /*
  *		============================
@@ -593,8 +674,15 @@ Datum
 int4um(PG_FUNCTION_ARGS)
 {
 	int32		arg = PG_GETARG_INT32(0);
+	int32		result;
 
-	PG_RETURN_INT32(-arg);
+	result = -arg;
+	/* overflow check (needed for INT_MIN) */
+	if (arg != 0 && SAMESIGN(result, arg))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -610,8 +698,20 @@ int4pl(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int32		arg2 = PG_GETARG_INT32(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 + arg2);
+	result = arg1 + arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of different signs then their sum
+	 * cannot overflow.  If the inputs are of the same sign, their sum had
+	 * better be that sign too.
+	 */
+	if (SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -619,8 +719,20 @@ int4mi(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int32		arg2 = PG_GETARG_INT32(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 - arg2);
+	result = arg1 - arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of the same sign then their
+	 * difference cannot overflow.	If they are of different signs then the
+	 * result should be of the same sign as the first input.
+	 */
+	if (!SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -628,8 +740,40 @@ int4mul(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int32		arg2 = PG_GETARG_INT32(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 * arg2);
+#ifdef WIN32
+	/*
+	 *	Win32 doesn't throw a catchable exception for
+	 *	SELECT -2147483648 * (-1); -- INT_MIN
+	 */
+	if (arg2 == -1 && arg1 == INT_MIN)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+#endif
+
+	result = arg1 * arg2;
+
+	/*
+	 * Overflow check.	We basically check to see if result / arg2 gives arg1
+	 * again.  There are two cases where this fails: arg2 = 0 (which cannot
+	 * overflow) and arg1 = INT_MIN, arg2 = -1 (where the division itself will
+	 * overflow and thus incorrectly match).
+	 *
+	 * Since the division is likely much more expensive than the actual
+	 * multiplication, we'd like to skip it where possible.  The best bang for
+	 * the buck seems to be to check whether both inputs are in the int16
+	 * range; if so, no overflow is possible.
+	 */
+	if (!(arg1 >= (int32) SHRT_MIN && arg1 <= (int32) SHRT_MAX &&
+		  arg2 >= (int32) SHRT_MIN && arg2 <= (int32) SHRT_MAX) &&
+		arg2 != 0 &&
+		(result / arg2 != arg1 || (arg2 == -1 && arg1 < 0 && result < 0)))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -637,29 +781,67 @@ int4div(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int32		arg2 = PG_GETARG_INT32(1);
+	int32		result;
 
 	if (arg2 == 0)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
 
-	PG_RETURN_INT32(arg1 / arg2);
+#ifdef WIN32
+	/*
+	 *	Win32 doesn't throw a catchable exception for
+	 *	SELECT -2147483648 / (-1); -- INT_MIN
+	 */
+	if (arg2 == -1 && arg1 == INT_MIN)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+#endif
+
+	result = arg1 / arg2;
+
+	/*
+	 * Overflow check.	The only possible overflow case is for arg1 = INT_MIN,
+	 * arg2 = -1, where the correct result is -INT_MIN, which can't be
+	 * represented on a two's-complement machine.
+	 */
+	if (arg2 == -1 && arg1 < 0 && result < 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
 int4inc(PG_FUNCTION_ARGS)
 {
 	int32		arg = PG_GETARG_INT32(0);
+	int32		result;
 
-	PG_RETURN_INT32(arg + 1);
+	result = arg + 1;
+	/* Overflow check */
+	if (arg > 0 && result < 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+
+	PG_RETURN_INT32(result);
 }
 
 Datum
 int2um(PG_FUNCTION_ARGS)
 {
 	int16		arg = PG_GETARG_INT16(0);
+	int16		result;
 
-	PG_RETURN_INT16(-arg);
+	result = -arg;
+	/* overflow check (needed for SHRT_MIN) */
+	if (arg != 0 && SAMESIGN(result, arg))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
+	PG_RETURN_INT16(result);
 }
 
 Datum
@@ -675,8 +857,20 @@ int2pl(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int16		result;
 
-	PG_RETURN_INT16(arg1 + arg2);
+	result = arg1 + arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of different signs then their sum
+	 * cannot overflow.  If the inputs are of the same sign, their sum had
+	 * better be that sign too.
+	 */
+	if (SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
+	PG_RETURN_INT16(result);
 }
 
 Datum
@@ -684,8 +878,20 @@ int2mi(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int16		result;
 
-	PG_RETURN_INT16(arg1 - arg2);
+	result = arg1 - arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of the same sign then their
+	 * difference cannot overflow.	If they are of different signs then the
+	 * result should be of the same sign as the first input.
+	 */
+	if (!SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
+	PG_RETURN_INT16(result);
 }
 
 Datum
@@ -693,8 +899,20 @@ int2mul(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int32		result32;
 
-	PG_RETURN_INT16(arg1 * arg2);
+	/*
+	 * The most practical way to detect overflow is to do the arithmetic in
+	 * int32 (so that the result can't overflow) and then do a range check.
+	 */
+	result32 = (int32) arg1 *(int32) arg2;
+
+	if (result32 < SHRT_MIN || result32 > SHRT_MAX)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
+
+	PG_RETURN_INT16((int16) result32);
 }
 
 Datum
@@ -702,13 +920,25 @@ int2div(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int16		result;
 
 	if (arg2 == 0)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
 
-	PG_RETURN_INT16(arg1 / arg2);
+	result = arg1 / arg2;
+
+	/*
+	 * Overflow check.	The only possible overflow case is for arg1 =
+	 * SHRT_MIN, arg2 = -1, where the correct result is -SHRT_MIN, which can't
+	 * be represented on a two's-complement machine.
+	 */
+	if (arg2 == -1 && arg1 < 0 && result < 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
+	PG_RETURN_INT16(result);
 }
 
 Datum
@@ -716,8 +946,20 @@ int24pl(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
 	int32		arg2 = PG_GETARG_INT32(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 + arg2);
+	result = arg1 + arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of different signs then their sum
+	 * cannot overflow.  If the inputs are of the same sign, their sum had
+	 * better be that sign too.
+	 */
+	if (SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -725,8 +967,20 @@ int24mi(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
 	int32		arg2 = PG_GETARG_INT32(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 - arg2);
+	result = arg1 - arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of the same sign then their
+	 * difference cannot overflow.	If they are of different signs then the
+	 * result should be of the same sign as the first input.
+	 */
+	if (!SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -734,8 +988,26 @@ int24mul(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
 	int32		arg2 = PG_GETARG_INT32(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 * arg2);
+	result = arg1 * arg2;
+
+	/*
+	 * Overflow check.	We basically check to see if result / arg2 gives arg1
+	 * again.  There is one case where this fails: arg2 = 0 (which cannot
+	 * overflow).
+	 *
+	 * Since the division is likely much more expensive than the actual
+	 * multiplication, we'd like to skip it where possible.  The best bang for
+	 * the buck seems to be to check whether both inputs are in the int16
+	 * range; if so, no overflow is possible.
+	 */
+	if (!(arg2 >= (int32) SHRT_MIN && arg2 <= (int32) SHRT_MAX) &&
+		result / arg2 != arg1)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -748,8 +1020,8 @@ int24div(PG_FUNCTION_ARGS)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
-
-	PG_RETURN_INT32(arg1 / arg2);
+	/* No overflow is possible */
+	PG_RETURN_INT32((int32) arg1 / arg2);
 }
 
 Datum
@@ -757,8 +1029,20 @@ int42pl(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 + arg2);
+	result = arg1 + arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of different signs then their sum
+	 * cannot overflow.  If the inputs are of the same sign, their sum had
+	 * better be that sign too.
+	 */
+	if (SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -766,8 +1050,20 @@ int42mi(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 - arg2);
+	result = arg1 - arg2;
+
+	/*
+	 * Overflow check.	If the inputs are of the same sign then their
+	 * difference cannot overflow.	If they are of different signs then the
+	 * result should be of the same sign as the first input.
+	 */
+	if (!SAMESIGN(arg1, arg2) && !SAMESIGN(result, arg1))
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -775,8 +1071,26 @@ int42mul(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int32		result;
 
-	PG_RETURN_INT32(arg1 * arg2);
+	result = arg1 * arg2;
+
+	/*
+	 * Overflow check.	We basically check to see if result / arg1 gives arg2
+	 * again.  There is one case where this fails: arg1 = 0 (which cannot
+	 * overflow).
+	 *
+	 * Since the division is likely much more expensive than the actual
+	 * multiplication, we'd like to skip it where possible.  The best bang for
+	 * the buck seems to be to check whether both inputs are in the int16
+	 * range; if so, no overflow is possible.
+	 */
+	if (!(arg1 >= (int32) SHRT_MIN && arg1 <= (int32) SHRT_MAX) &&
+		result / arg1 != arg2)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -784,13 +1098,25 @@ int42div(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
 	int16		arg2 = PG_GETARG_INT16(1);
+	int32		result;
 
 	if (arg2 == 0)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
 
-	PG_RETURN_INT32(arg1 / arg2);
+	result = arg1 / arg2;
+
+	/*
+	 * Overflow check.	The only possible overflow case is for arg1 = INT_MIN,
+	 * arg2 = -1, where the correct result is -INT_MIN, which can't be
+	 * represented on a two's-complement machine.
+	 */
+	if (arg2 == -1 && arg1 < 0 && result < 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
@@ -803,6 +1129,7 @@ int4mod(PG_FUNCTION_ARGS)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
+	/* No overflow is possible */
 
 	PG_RETURN_INT32(arg1 % arg2);
 }
@@ -817,6 +1144,7 @@ int2mod(PG_FUNCTION_ARGS)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
+	/* No overflow is possible */
 
 	PG_RETURN_INT16(arg1 % arg2);
 }
@@ -831,6 +1159,7 @@ int24mod(PG_FUNCTION_ARGS)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
+	/* No overflow is possible */
 
 	PG_RETURN_INT32(arg1 % arg2);
 }
@@ -845,44 +1174,11 @@ int42mod(PG_FUNCTION_ARGS)
 		ereport(ERROR,
 				(errcode(ERRCODE_DIVISION_BY_ZERO),
 				 errmsg("division by zero")));
+	/* No overflow is possible */
 
 	PG_RETURN_INT32(arg1 % arg2);
 }
 
-/* int[24]fac()
- * Factorial
- */
-Datum
-int4fac(PG_FUNCTION_ARGS)
-{
-	int32		arg1 = PG_GETARG_INT32(0);
-	int32		result;
-
-	if (arg1 == 0)
-		result = 1;
-	else if (arg1 < 0)
-		result = 0;
-	else
-		for (result = 1; arg1 > 0; --arg1)
-			result *= arg1;
-	PG_RETURN_INT32(result);
-}
-
-Datum
-int2fac(PG_FUNCTION_ARGS)
-{
-	int16		arg1 = PG_GETARG_INT16(0);
-	int32		result;
-
-	if (arg1 == 0)
-		result = 1;
-	else if (arg1 < 0)
-		result = 0;
-	else
-		for (result = 1; arg1 > 0; --arg1)
-			result *= arg1;
-	PG_RETURN_INT32(result);
-}
 
 /* int[24]abs()
  * Absolute value
@@ -891,16 +1187,30 @@ Datum
 int4abs(PG_FUNCTION_ARGS)
 {
 	int32		arg1 = PG_GETARG_INT32(0);
+	int32		result;
 
-	PG_RETURN_INT32((arg1 < 0) ? -arg1 : arg1);
+	result = (arg1 < 0) ? -arg1 : arg1;
+	/* overflow check (needed for INT_MIN) */
+	if (result < 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("integer out of range")));
+	PG_RETURN_INT32(result);
 }
 
 Datum
 int2abs(PG_FUNCTION_ARGS)
 {
 	int16		arg1 = PG_GETARG_INT16(0);
+	int16		result;
 
-	PG_RETURN_INT16((arg1 < 0) ? -arg1 : arg1);
+	result = (arg1 < 0) ? -arg1 : arg1;
+	/* overflow check (needed for SHRT_MIN) */
+	if (result < 0)
+		ereport(ERROR,
+				(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+				 errmsg("smallint out of range")));
+	PG_RETURN_INT16(result);
 }
 
 Datum
@@ -939,7 +1249,8 @@ int4smaller(PG_FUNCTION_ARGS)
 	PG_RETURN_INT32((arg1 < arg2) ? arg1 : arg2);
 }
 
-/* Binary arithmetics
+/*
+ * Bit-pushing operators
  *
  *		int[24]and		- returns arg1 & arg2
  *		int[24]or		- returns arg1 | arg2
@@ -1055,3 +1366,81 @@ int2shr(PG_FUNCTION_ARGS)
 
 	PG_RETURN_INT16(arg1 >> arg2);
 }
+
+/*
+ * non-persistent numeric series generator
+ */
+Datum
+generate_series_int4(PG_FUNCTION_ARGS)
+{
+	return generate_series_step_int4(fcinfo);
+}
+
+Datum
+generate_series_step_int4(PG_FUNCTION_ARGS)
+{
+	FuncCallContext *funcctx;
+	generate_series_fctx *fctx;
+	int32		result;
+	MemoryContext oldcontext;
+
+	/* stuff done only on the first call of the function */
+	if (SRF_IS_FIRSTCALL())
+	{
+		int32		start = PG_GETARG_INT32(0);
+		int32		finish = PG_GETARG_INT32(1);
+		int32		step = 1;
+
+		/* see if we were given an explicit step size */
+		if (PG_NARGS() == 3)
+			step = PG_GETARG_INT32(2);
+		if (step == 0)
+			ereport(ERROR,
+					(errcode(ERRCODE_INVALID_PARAMETER_VALUE),
+					 errmsg("step size may not equal zero")));
+
+		/* create a function context for cross-call persistence */
+		funcctx = SRF_FIRSTCALL_INIT();
+
+		/*
+		 * switch to memory context appropriate for multiple function calls
+		 */
+		oldcontext = MemoryContextSwitchTo(funcctx->multi_call_memory_ctx);
+
+		/* allocate memory for user context */
+		fctx = (generate_series_fctx *) palloc(sizeof(generate_series_fctx));
+
+		/*
+		 * Use fctx to keep state from call to call. Seed current with the
+		 * original start value
+		 */
+		fctx->current = start;
+		fctx->finish = finish;
+		fctx->step = step;
+
+		funcctx->user_fctx = fctx;
+		MemoryContextSwitchTo(oldcontext);
+	}
+
+	/* stuff done on every call of the function */
+	funcctx = SRF_PERCALL_SETUP();
+
+	/*
+	 * get the saved state and use current as the result for this iteration
+	 */
+	fctx = funcctx->user_fctx;
+	result = fctx->current;
+
+	if ((fctx->step > 0 && fctx->current <= fctx->finish) ||
+		(fctx->step < 0 && fctx->current >= fctx->finish))
+	{
+		/* increment current in preparation for next iteration */
+		fctx->current += fctx->step;
+
+		/* do when there is more left to send */
+		SRF_RETURN_NEXT(funcctx, Int32GetDatum(result));
+	}
+	else
+		/* do when there is no more left */
+		SRF_RETURN_DONE(funcctx);
+}
